@@ -78,25 +78,80 @@ let under_ctx name ty bo gterm2lp ~depth state x =
           state, mk_def ~depth name ~bo:(lift1 bo) ~ty:(lift1 ty) in
     let new_hyp = { ctx_entry; depth = depth+1 } in
     set_coq_ctx_hyps state ({ coq_ctx with name2db }, new_hyp :: hyps), Name id in
-  let state, y = gterm2lp ~depth:(depth+1) (push_env state name) x in
+  let state, y, gl = gterm2lp ~depth:(depth+1) (push_env state name) x in
   let state = set_coq_ctx_hyps state orig_ctx in
   let state = pop_env state in
-  state, y
+  state, y, gl
 
 let type_gen = ref 0
 
 let is_hole x = match DAst.get x with GHole _ -> true | _ -> false
 
+let universe_level_name evd ({CAst.v=id} as lid) =
+  try Evd.universe_of_name evd id
+  with Not_found ->
+    CErrors.user_err ?loc:lid.CAst.loc
+      (Pp.(str "Undeclared universe: " ++ Id.print id ++ str "."))
+
+let sort_name env sigma l = match l with
+| [] -> assert false
+| [u, 0] ->
+  begin match u with
+  | GSet -> Sorts.set
+  | GSProp -> Sorts.sprop
+  | GProp -> Sorts.prop
+  | GUniv u -> Sorts.sort_of_univ (Univ.Universe.make u)
+  | GLocalUniv l ->
+    let u = universe_level_name sigma l in
+    Sorts.sort_of_univ (Univ.Universe.make u)
+  | GRawUniv _ -> nYI "GRawUniv"
+  end
+| [_] | _ :: _ :: _ ->
+  nYI "(glob)HOAS for Type@{i j}"
+
+let glob_level loc state = function
+  | UAnonymous _ -> nYI "UAnonymous"
+  | UNamed s ->
+      match s with
+      | GSProp 
+      | GProp ->
+        CErrors.user_err ?loc
+          Pp.(str "Universe instances cannot contain non-Set small levels, polymorphic" ++
+              str " universe instances must be greater or equal to Set.");
+      | GSet -> Univ.Level.set
+      | GUniv u -> u
+      | GRawUniv u -> nYI "GRawUniv"
+      | GLocalUniv l -> universe_level_name (get_sigma state) l
+
+let nogls f ~depth state x = let state, x = f ~depth state x in state, x, ()
+let noglsk f ~depth state = let state, x = f ~depth state in state, x, ()
+
 let rec gterm2lp ~depth state x =
   debug Pp.(fun () ->
       str"gterm2lp: depth=" ++ int depth ++
       str " term=" ++Printer.pr_glob_constr_env (get_global_env state) (get_sigma state) x);
   match (DAst.get x) (*.CAst.v*) with
+  | GRef(GlobRef.ConstRef p,_ul) when Structures.PrimitiveProjections.mem p ->
+      let p = Option.get @@ Structures.PrimitiveProjections.find_opt p in
+      let hd = in_elpi_gr ~depth state (GlobRef.ConstRef (Projection.Repr.constant p)) in
+      state, hd
+  | GRef(gr, ul) when Global.is_polymorphic gr ->
+    begin match ul with
+    | None ->
+      incr type_gen;
+      let state, s =
+        API.RawQuery.mk_Arg state ~name:(Printf.sprintf "univ_inst_%d" !type_gen) ~args:[]
+      in
+      state, in_elpi_poly_gr ~depth state gr s
+    | Some l -> 
+      let l' = List.map (glob_level x.CAst.loc state) l in
+      state, in_elpi_poly_gr_instance ~depth state gr (Univ.Instance.of_array (Array.of_list l'))
+    end
   | GRef(gr,_ul) -> state, in_elpi_gr ~depth state gr
   | GVar(id) ->
       let ctx, _ = Option.default (upcast @@ mk_coq_context ~options:default_options state, []) (get_ctx state) in
       if not (Id.Map.mem id ctx.name2db) then
-        CErrors.user_err
+        CErrors.user_err ?loc:x.CAst.loc
           Pp.(str"Free Coq variable " ++ Names.Id.print id ++ str " in context: " ++
             prlist_with_sep spc Id.print (Id.Map.bindings ctx.name2db |> List.map fst));
       state, E.mkConst (Id.Map.find id ctx.name2db)
@@ -104,18 +159,18 @@ let rec gterm2lp ~depth state x =
       incr type_gen;
       let state, s = API.RawQuery.mk_Arg state ~name:(Printf.sprintf "type_%d" !type_gen) ~args:[] in
       state, in_elpi_flex_sort s
-  | GSort(UNamed [name,0]) ->
-    let env = get_global_env state in
-      state, in_elpi_sort (Sorts.sort_of_univ @@ Univ.Universe.make @@ Pretyping.known_glob_level (Evd.from_env env) name)
+  | GSort(UNamed u) ->
+      let env = get_global_env state in
+      in_elpi_sort ~depth state (sort_name env (get_sigma state) u)
   | GSort(_) -> nYI "(glob)HOAS for Type@{i j}"
 
   | GProd(name,_,s,t) ->
       let state, s = gterm2lp ~depth state s in
-      let state, t = under_ctx name s None gterm2lp ~depth state t in
+      let state, t, () = under_ctx name s None (nogls gterm2lp) ~depth state t in
       state, in_elpi_prod name s t
   | GLambda(name,_,s,t) ->
       let state, s = gterm2lp ~depth state s in
-      let state, t = under_ctx name s None gterm2lp ~depth state t in
+      let state, t, () = under_ctx name s None (nogls gterm2lp) ~depth state t in
       state, in_elpi_lam name s t
   | GLetIn(name,bo , oty, t) ->
       let state, bo = gterm2lp ~depth state bo in
@@ -127,7 +182,7 @@ let rec gterm2lp ~depth state x =
             let args = List.map (fun (_,x) -> E.mkBound x) (Id.Map.bindings ctx.name2db) in
             state, E.mkUnifVar uv ~args state
         | Some ty -> gterm2lp ~depth state ty in
-      let state, t = under_ctx name ty (Some bo) gterm2lp ~depth state t in
+      let state, t, () = under_ctx name ty (Some bo) (nogls gterm2lp) ~depth state t in
       state, in_elpi_let name bo ty t
 
   | GHole(_,_,Some arg) when !is_elpi_code arg ->
@@ -305,7 +360,7 @@ let rec gterm2lp ~depth state x =
       let ty = glob_intros_prod tctx ty in
       let state, ty = gterm2lp ~depth state ty in
       let bo = glob_intros tctx bo in
-      let state, bo = under_ctx (Name name) ty None gterm2lp ~depth state bo in
+      let state, bo, () = under_ctx (Name name) ty None (nogls gterm2lp) ~depth state bo in
       state, in_elpi_fix (Name name) rno ty bo
   | GRec _ -> nYI "(glob)HOAS mutual/non-struct fix"
   | GInt i -> in_elpi_primitive ~depth state (Uint63 i)
@@ -313,9 +368,23 @@ let rec gterm2lp ~depth state x =
   | GArray _ -> nYI "(glob)HOAS persistent arrays"
 ;;
 
-let coq_quotation ~depth state _loc src =
-  let ce = Pcoq.parse_string Pcoq.Constr.lconstr src in
-  gterm2lp ~depth state (Constrintern.intern_constr (get_global_env state) (get_sigma state) ce)
+let coq_quotation ~depth state loc src =
+  let ce =
+    try
+      Pcoq.parse_string ~loc:(to_coq_loc loc) Pcoq.Constr.lconstr src
+    with e ->
+      CErrors.user_err
+        Pp.(str(API.Ast.Loc.show loc) ++ spc() ++ CErrors.print_no_report e)
+  in
+  let glob =
+    try
+      Constrintern.intern_constr (get_global_env state) (get_sigma state) ce
+    with e ->
+      CErrors.user_err
+        Pp.(str(API.Ast.Loc.show loc) ++str":" ++ spc() ++ CErrors.print_no_report e)
+  in
+  gterm2lp ~depth state glob
+      
 
 (* Install the quotation *)
 let () = Q.set_default_quotation coq_quotation
@@ -340,7 +409,7 @@ let rec do_params params kont ~depth state =
   | (name,imp,ob,src) :: params ->
       if ob <> None then Coq_elpi_utils.nYI "defined parameters in a record/inductive declaration";
       let state, src = gterm2lp ~depth state src in
-      let state, tgt = under_ctx name src None (do_params params kont) ~depth state in
+      let state, tgt, () = under_ctx name src None (noglsk (do_params params kont)) ~depth state in
       let state, imp = in_elpi_imp ~depth state imp in
       state, in_elpi_parameter name ~imp src tgt
 
@@ -353,7 +422,7 @@ let rec do_fields fields ~depth state =
   | [] -> state, in_elpi_indtdecl_endrecord ()
   | (f,({ name; is_coercion; is_canonical } as att)) :: fields ->
       let state, f = do_term f ~depth state in
-      let state, fields = under_ctx name f None (do_fields fields) ~depth state in
+      let state, fields, () = under_ctx name f None (noglsk (do_fields fields)) ~depth state in
       in_elpi_indtdecl_field ~depth state att f fields
 
 let do_record ~name ~constructorname arity fields ~depth state =
