@@ -104,14 +104,18 @@ let manual_implicit_of_binding_kind name = function
   | Glob_term.MaxImplicit -> CAst.make (Some (name,true))
   | Glob_term.Explicit -> CAst.make None
 
+let binding_kind_of_manual_implicit x =
+  match x.CAst.v with
+  | Some (_,false) -> Glob_term.NonMaxImplicit
+  | Some (_,true) -> Glob_term.MaxImplicit
+  | None -> Glob_term.Explicit
+
 let manual_implicit_of_gdecl (name,bk,_,_) = manual_implicit_of_binding_kind name bk
 
 let lookup_inductive env i =
   let mind, indbo = Inductive.lookup_mind_specif env i in
   if Array.length mind.Declarations.mind_packets <> 1 then
     nYI "API(env) mutual inductive";
-  if Declareops.inductive_is_polymorphic mind then
-    nYI "API(env) poly mutual inductive";
   mind, indbo
 
 
@@ -119,8 +123,8 @@ let locate_qualid qualid =
   try
     match Nametab.locate_extended qualid with
     | Globnames.TrueGlobal gr -> Some (`Gref gr)
-    | Globnames.SynDef sd ->
-       match Syntax_def.search_syntactic_definition sd with
+    | Globnames.Abbrev sd ->
+       match Abbreviation.search_abbreviation sd with
        | _, Notation_term.NRef(gr,_) -> Some (`Gref gr)
        | _ -> Some (`Abbrev sd)
   with Not_found -> None
@@ -212,11 +216,13 @@ let rec list_map_acc f acc = function
 let rec fix_detype x = match DAst.get x with
   | Glob_term.GEvar _ -> mkGHole
   | _ -> Glob_ops.map_glob_constr fix_detype x
-let detype env sigma t =
+let detype ?(keepunivs=false) env sigma t =
   (* To avoid turning named universes into unnamed ones *)
+  let options =
+    if keepunivs then Flags.with_option Constrextern.print_universes
+    else (fun f x -> f x) in
   let gbody =
-    Flags.with_option Constrextern.print_universes
-      (Detyping.detype Detyping.Now false Names.Id.Set.empty env sigma) t in
+    options (Detyping.detype Detyping.Now false Names.Id.Set.empty env sigma) t in
   fix_detype gbody
 
 let detype_closed_glob env sigma closure =
@@ -228,3 +234,37 @@ let compare_qualified_name = Stdlib.compare
 let pr_qualified_name = Pp.prlist_with_sep (fun () -> Pp.str".") Pp.str
 let show_qualified_name = String.concat "."
 let pp_qualified_name fmt l = Format.fprintf fmt "%s" (String.concat "." l)
+
+let option_map_acc f s = function
+  | None -> s, None
+  | Some x ->
+      let s, x = f s x in
+      s, Some x
+
+let option_map_acc2 f s = function
+  | None -> s, None, []
+  | Some x ->
+      let s, x, gl = f s x in
+      s, Some x, gl
+    
+let option_default f = function
+  | Some x -> x
+  | None -> f ()
+
+let coq_version_parser version =
+  let (!!) x = try int_of_string x with Failure _ -> -100 in
+  match Str.split (Str.regexp_string ".") version with
+  | major :: minor :: patch :: _ -> !!major, !!minor, !!patch
+  | [ major ] -> !!major,0,0
+  | [] -> 0,0,0
+  | [ major; minor ] ->
+      match Str.split (Str.regexp_string "+") minor with
+      | [ minor ] -> !!major, !!minor, 0
+      | [ ] -> !!major, !!minor, 0
+      | minor :: prerelease :: _ ->
+          if Str.string_match (Str.regexp_string "beta") prerelease 0 then
+            !!major, !!minor, !!("-"^String.sub prerelease 4 (String.length prerelease - 4))
+          else if Str.string_match (Str.regexp_string "alpha") prerelease 0 then
+            !!major, !!minor, !!("-"^String.sub prerelease 5 (String.length prerelease - 5))
+          else !!major, !!minor, -100
+
